// openwithnotepadShl.cpp : Implementation of CopenwithnotepadShl
#include "stdafx.h"
#include <atlconv.h>  // for ATL string conversion macros

#include "openwithnotepad.h"
#include "openwithnotepadShl.h"

//#define DEBUG_ADDIN
#ifdef DEBUG_ADDIN
#define TRACE(s)	::OutputDebugString(s)
#else
#define TRACE(s)
#endif

#define NOTEPAD		_T("notepad.exe ")	// Note the space

/////////////////////////////////////////////////////////////////////////////
// CopenwithnotepadShl

CopenwithnotepadShl::CopenwithnotepadShl()
{
	m_hNotepadBmp = LoadBitmap(_Module.GetModuleInstance(), MAKEINTRESOURCE(IDB_NOTEPAD));
}


HRESULT CopenwithnotepadShl::Initialize(
    LPCITEMIDLIST pidlFolder,
    LPDATAOBJECT  pDataObj,
    HKEY          hProgID
	)
{
	UNREFERENCED_PARAMETER(pidlFolder);
	UNREFERENCED_PARAMETER(hProgID);

	FORMATETC fmt = { CF_HDROP, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
	STGMEDIUM stg = { TYMED_HGLOBAL };
	HDROP     hDrop;

	TRACE(_T("Open with Notepad: initialize\n"));

    // Look for CF_HDROP data in the data object
    if (FAILED(pDataObj->GetData(&fmt, &stg)))
	{
        // Nope! Return an "invalid argument" error back to Explorer
		TRACE(_T("Open with Notepad: invalid arg in initialize\n"));
        return E_INVALIDARG;
	}

    // Get a pointer to the actual data
    hDrop = (HDROP)GlobalLock(stg.hGlobal);
    if (NULL == hDrop)
	{
        ReleaseStgMedium(&stg);
        return E_INVALIDARG;
	}

    // Sanity check	- make sure there is at least one filename
	UINT uNumFiles = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);
    if (0 == uNumFiles)
	{
        GlobalUnlock(stg.hGlobal);
        ReleaseStgMedium(&stg);
        return E_INVALIDARG;
	}

	TCHAR szFile[MAX_PATH];
    for (UINT uFile = 0 ; uFile < uNumFiles ; uFile++)
	{
        // Get the next filename.
        if (0 == DragQueryFile(hDrop, uFile, szFile, MAX_PATH))
            continue;

        // Add the file name to our list of files (m_lsFiles).
        m_lsFiles.push_back(szFile);
	}

    GlobalUnlock(stg.hGlobal);
    ReleaseStgMedium(&stg);

    // If we found any files we can work with, return S_OK. Otherwise, return
    // E_INVALIDARG so we don't get called again for this right-click operation.
    return (m_lsFiles.size() > 0) ? S_OK : E_INVALIDARG;
}


HRESULT CopenwithnotepadShl::QueryContextMenu(
    HMENU hmenu,
    UINT  uMenuIndex, 
    UINT  uidFirstCmd,
    UINT  uidLastCmd,
    UINT  uFlags
	)
{
	UNREFERENCED_PARAMETER(uidLastCmd);

    // If the flags include CMF_DEFAULTONLY then we shouldn't do anything
    if (uFlags & CMF_DEFAULTONLY)
	{
        return MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_NULL, 0);
	}

	UINT first = uidFirstCmd;

    // Insert the Send to notepad name menu item
    InsertMenu(hmenu, uMenuIndex, MF_STRING | MF_BYPOSITION, uidFirstCmd, _T("Open with Notepad"));
    if (NULL != m_hNotepadBmp)
    {
		SetMenuItemBitmaps(hmenu, uMenuIndex, MF_BYPOSITION, m_hNotepadBmp, NULL);
    }

	//uMenuIndex++;
	//uidFirstCmd++;

    return MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_NULL, uidFirstCmd - first + 1);
}


HRESULT CopenwithnotepadShl::GetCommandString(
    UINT_PTR  idCmd,
    UINT  uFlags,
    UINT* pwReserved,
    LPSTR pszName,
    UINT  cchMax
	)
{
	UNREFERENCED_PARAMETER(pwReserved);

	LPCTSTR szPrompt;
    USES_CONVERSION;

    // If Explorer is asking for a help string, copy our string into the
    // supplied buffer
    if (uFlags & GCS_HELPTEXT)
	{
		switch (idCmd)
		{
		case 0:  szPrompt = _T("Open the file(s) with Notepad for viewing");     break;
		default: return E_INVALIDARG;
		}

        if (uFlags & GCS_UNICODE)
		{
            // We need to cast pszName to a Unicode string, and then use the
            // Unicode string copy API
            lstrcpynW((LPWSTR)pszName, T2CW(szPrompt), cchMax);
		}
        else
		{
            // Use the ANSI string copy API to return the help string
            lstrcpynA(pszName, T2CA(szPrompt), cchMax);
		}

        return S_OK;
	}

    return E_INVALIDARG;
}


HRESULT CopenwithnotepadShl::InvokeCommand(LPCMINVOKECOMMANDINFO pCmdInfo)
{
    // If lpVerb really points to a string, ignore this function call and bail out
    if (0 != HIWORD(pCmdInfo->lpVerb))
        return E_INVALIDARG;

    // Get the command index - the only valid commands are 0
	WORD wCmd = LOWORD(pCmdInfo->lpVerb);
	if (wCmd > 0)
	{
		TRACE(_T("Open with Notepad: illegal invoke command\n"));
		return E_INVALIDARG;
	}

	if (wCmd == 0)
	{
		TRACE(_T("Open with Notepad: invoke command 0\n"));
		return SendToNotepad();
	}

	return S_OK;
}


HRESULT CopenwithnotepadShl::SendToNotepad()
{
	STARTUPINFO si;
    PROCESS_INFORMATION pi;
    ::ZeroMemory(&si, sizeof(si));
    si.cb = sizeof(si);
    ::ZeroMemory(&pi, sizeof(pi));

	string_list::const_iterator it;
	for (it = m_lsFiles.begin() ; it != m_lsFiles.end() ; it++)
	{
		TCHAR *cmdLine = new TCHAR[lstrlen(NOTEPAD) + lstrlen((LPTSTR)it->c_str()) + 1];
		lstrcpy(cmdLine, NOTEPAD);
		lstrcat(cmdLine, (LPTSTR)it->c_str());
		TRACE(_T("Open with Notepad: run "));
		TRACE(cmdLine);
		TRACE(_T("\n"));
		::CreateProcess(NULL, cmdLine, NULL, NULL, FALSE,
					CREATE_BREAKAWAY_FROM_JOB | CREATE_DEFAULT_ERROR_MODE,
					NULL, NULL, &si, &pi);
		::CloseHandle(pi.hProcess);
		::CloseHandle(pi.hThread);
		delete cmdLine;
	}
	return S_OK;
}
